"""
    Zope Static Call Tree Utility

    $Id: staticcalltree.py,v 1.1 2006/11/09 14:28:41 kevinadmin Exp $
"""

from Acquisition import aq_base
from DateTime import DateTime
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import ClassSecurityInfo, Role

from ZODB.PersistentMapping import PersistentMapping

from parsers import PARSERS, NONEXECUTABLEOBJECTS

# Use this to increase the data displayed when scanning
DEBUG=2
INFO=1

TRACE_LEVEL=1

security = ClassSecurityInfo()

class StaticCallTree(SimpleItem, PropertyManager ):
    """ Snapshot information about the callers and callee in the system.
    """
    meta_type = 'Zope Static Call Tree'

    security.declarePrivate('ConvertToFullPath')
    def ConvertToFullPath(self, obj, obj_path, target):
        """For a given target, what is the real path to that object,
        (from the static call tree path - self.context_path).

        obj_path is the path from the current context to the object.
        obj is the object
        target is the path to the target from the object
        """

        target_obj_name=target.split('/')[0]

        while obj:
            try:
                objectIds = obj.objectIds()
            except:
                objectIds = []

            if target_obj_name in objectIds:
                # OBJ is the acquired object. Need to
                # take the path of the acquired object and
                # add on the acquisition path. This path
                # must be made relative to the root folder

                # First get the path from current context to object
                path=[]
		for p in obj.getPhysicalPath(): 
		    if p: path.append(p)

                # prefix the path to the current context
                aq_path = '/'+'/'.join(path)
                rv = aq_path+'/'+target
                return rv

            try:
                obj = obj.aq_parent
            except:
                obj = None

        return "NOT FOUND: %s" % target

    security.declareProtected('Manage properties', 'ScanFolders')
    def ScanFolders(self, REQUEST):
        """Scan sub-folders and construct two dictionaries to track
        the data. objects has an entry per object found and parsed.
        targets has a list of callees and links back to the callers
        """
        self.objects.clear()
        self.targets.clear()
        self._p_changed = 1
        self.scanned = DateTime()

        rv=[] # Used for logging

        supported_meta_types=PARSERS.keys()
        scannedcount={}
        for s in supported_meta_types:
            scannedcount[s] = 0

        for p, obj in self.aq_parent.ZopeFind(self.aq_parent, search_sub=1 ):
            # Determine the content type. Apply a parser
            # and extract information about the object
            if obj.meta_type in supported_meta_types:
                scannedcount[obj.meta_type]=scannedcount[obj.meta_type]+1
                object_path=self.context_path + p
                if TRACE_LEVEL >= DEBUG: rv.append(str("PARSED: %s"% p))
                res = PARSERS[obj.meta_type](obj)
                if TRACE_LEVEL >= DEBUG: rv.append("RESULT:"+str(res))
                methods=[]
                for m in res[0]:
                    try:
                        fp = self.ConvertToFullPath(obj, p, m)
                        methods.append(fp)
                        if TRACE_LEVEL >= DEBUG: rv.append('method: %s -> %s' % (m, fp))

                        # add like to callers 
                        if fp not in self.targets.keys():
                            self.targets[fp] = []
                        if object_path not in self.targets[fp]:
                            self.targets[fp].append(object_path)

                    except:
                        rv.append('Exception Converting %s:%s to full path' % (p, m))

                self.objects[object_path] = (methods, res[1])
            else:
                if obj.meta_type not in NONEXECUTABLEOBJECTS:
                    if TRACE_LEVEL >= INFO:
                        rv.append('No parser for type [%s] (%s)' % (obj.meta_type, p))
        rv.append("\nSCAN COUNTERS:")
        for s in supported_meta_types:
            rv.append('\t%s: %s' % (s, scannedcount[s]))

        return '\n'.join(rv)

    def __init__( self, context ):
        """Create new object"""

        path = []
        for p in context.getPhysicalPath(): 
            if p: path.append(p)

        self.context_path = '/'+'/'.join( path )
        if len(self.context_path) > 1: self.context_path=self.context_path  +'/'

        self.created = DateTime()
        self.scanned = DateTime() # None gives an error with APE
        self.objects = {}
        self.targets = {}

    _properties = ( { 'id' : 'context_path', 'type' : 'string', 'mode' : 'r' },
                    { 'id' : 'created', 'type' : 'date', 'mode' : 'r' },
                    { 'id' : 'scanned', 'type' : 'date', 'mode' : 'r' },
                  )

    manage_options = ( { 'label' : 'Properties'
                       , 'action' : 'manage_propertiesForm'
                       },
                       { 'label' : 'View'
                       , 'action' : 'index_html'
                       },
                       { 'label' : 'Scan Folder'
                       , 'action' : 'ScanFolders'
                       },
                     )+ Role.RoleManager.manage_options

    security.declareProtected('Manage properties', 'index_html')
    index_html = PageTemplateFile( 'zmi/view.zpt', globals() )

InitializeClass(StaticCallTree)

def manage_addStaticCallTree(dispatcher, REQUEST=None):
    """ Add a StaticCallTree
    """
    id =  'zopeStaticCallTree' # ensures that it is at the end
    context = getattr( aq_base( dispatcher ), '_d', dispatcher )
    calltree = StaticCallTree( context )
    calltree._setId( id )
    dispatcher._setObject( id, calltree )

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( '%s/manage_main'
              % dispatcher.absolute_url() )
